'use strict';

var createReactClass = require('create-react-class');
var PropTypes = require('prop-types');
var React = require('react');
var ReactDOM = require('react-dom');
var _ = require('lodash');

var iconClasses = {
    close: 'fa fa-close'
};

module.exports = createReactClass({
    displayName: 'Modal',

    propTypes: {
        autoFocusModal: PropTypes.bool,
        closeModalCallback: PropTypes.func,
        backgroundClickToClose: PropTypes.bool,
        footerButtonCallback: function footerButtonCallback(props) {
            if (props.footerButtonCallback && (typeof props.footerButtonCallback !== 'function' || !props.footerButtonText)) {
                return new Error('footerButtonCallback must be a function if included, and footerButtonText must be used with it.');
            }
        },
        footerButtonText: function footerButtonText(props) {
            if (props.footerButtonText && (typeof props.footerButtonText !== 'string' || !props.footerButtonCallback)) {
                return new Error('footerButtonText must be a string if included, and footerButtonCallback must be used with it.');
            }
        },
        iconClasses: PropTypes.object,
        showCloseIcon: PropTypes.bool,
        title: PropTypes.string
    },

    getDefaultProps: function getDefaultProps() {
        return {
            autoFocusModal: true,
            backgroundClickToClose: true,
            showCloseIcon: true
        };
    },

    getInitialState: function getInitialState() {
        this.iconClasses = _.merge(_.clone(iconClasses), this.props.iconClasses);

        return {};
    },

    /**
     * Draws focus to the content rendered within the modal once the component mounts.
     */
    componentDidMount: function componentDidMount() {
        if (this.props.autoFocusModal) {
            this.refs.content.focus();
        }
    },

    /**
     * We only want the component to rerender if the content to be rendered has changed. This will prevent a non-render
     * related prop change from swapping out input fields which causes a loss of focus while typing.
     * @param {object}   nextProps The next set of properties to be received by the component
     * @return {boolean}           Whether the component should update or not
     */
    shouldComponentUpdate: function shouldComponentUpdate(nextProps) {
        return nextProps.children !== this.props.children;
    },


    /**
     * Draws focus to the content rendered within the modal when the content changes. This generally happens with
     * modals that are reused for multi-step modal functionality.
     */
    componentDidUpdate: function componentDidUpdate() {
        if (this.props.autoFocusModal) {
            this.refs.content.focus();
        }
    },

    /**
     * Gets markup to display close icon in upper right corner. Only returns markup if
     * the showCloseIcon prop is set.
     * @return {null|ReactElement} Icon markup or null
     */
    getCloseIconMarkup: function getCloseIconMarkup() {
        if (!this.props.showCloseIcon) {
            return null;
        }
        return React.createElement(
            'span',
            { className: 'close', onClick: this.closeModalHandler },
            React.createElement(
                'span',
                { className: 'close-text' },
                'esc to close'
            ),
            ' | ',
            React.createElement('i', { className: this.iconClasses.close })
        );
    },

    /**
     * Returns the markup for the footer containing a button if the footerButtonText was passed in on props.
     * @return {null|ReactElement} Footer markup
     */
    getFooter: function getFooter() {
        if (!this.props.footerButtonText) {
            return null;
        }
        return React.createElement(
            'div',
            { className: 'footer' },
            React.createElement(
                'button',
                { onClick: this.props.footerButtonCallback },
                this.props.footerButtonText
            )
        );
    },


    render: function render() {
        return React.createElement(
            'div',
            { onClick: this.backgroundClickHandler, id: 'modal-container', 'data-clickcatcher': 'true' },
            React.createElement(
                'div',
                { ref: 'content', className: 'content', tabIndex: '-1', onKeyDown: this.keyDownHandler },
                React.createElement(
                    'div',
                    { className: 'header' },
                    React.createElement(
                        'span',
                        { className: 'title' },
                        this.props.title
                    ),
                    this.getCloseIconMarkup()
                ),
                React.createElement(
                    'div',
                    { className: 'body' },
                    this.props.children
                ),
                this.getFooter()
            )
        );
    },

    /**
     * If the key pressed is the escape key and the close icon is being shown, the close modal handler will be called.
     * @param {object} e - The simulated React event.
     */
    keyDownHandler: function keyDownHandler(e) {
        // escape key pressed
        if (e.keyCode === 27 && this.props.showCloseIcon) {
            this.closeModalHandler(e);
        }
    },

    /**
     * Captures any click event outside the modal and calls the close modal handler.
     * @param {Object} e - The simulated React event.
     */
    backgroundClickHandler: function backgroundClickHandler(e) {
        if (this.props.backgroundClickToClose && e.target.getAttribute('data-clickcatcher')) {
            this.closeModalHandler(e);
        }
    },

    /**
     * Triggered when clicking outside the modal, clicking on the close button, and when pressing escape.
     * @param {Object} e - The simulated React event.
     */
    closeModalHandler: function closeModalHandler(e) {
        e.stopPropagation();

        if (typeof this.props.closeModalCallback === 'function') {
            this.props.closeModalCallback();
        } else {
            ReactDOM.unmountComponentAtNode(this.parentNode);
        }
    }
});